package com.example.demo;

import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
	private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

	private final GenreService genreService;
	private final BookService bookService;

	public DemoApplication(GenreService genreService, BookService bookService) {
		this.genreService = genreService;
		this.bookService = bookService;
	}

	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	@Override
	public void run(String... args) throws Exception {
		if (args.length > 0 && Objects.equals("--populate", args[0])) {
			log.info("Create default genres values");
			final var genre1 = genreService.create(new GenreEntity(null, "Роман"));
			final var genre2 = genreService.create(new GenreEntity(null, "Фэнтези"));
			final var genre3 = genreService.create(new GenreEntity(null, "Трагедия"));

			log.info("Create default books values");
			bookService.create(new BookEntity(null, genre1, "g", "g", "g", "g"));
			bookService.create(new BookEntity(null, genre1, "q", "y", "q", "y"));
			bookService.create(new BookEntity(null, genre2, "w", "t", "w", "t"));
			bookService.create(new BookEntity(null, genre2, "e", "r", "e", "r"));
			bookService.create(new BookEntity(null, genre3, "r", "e", "r", "e"));
			bookService.create(new BookEntity(null, genre3, "t", "w", "t", "w"));
			bookService.create(new BookEntity(null, genre1, "y", "q", "y", "q"));
		}
	}
}
