package com.example.demo.books.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.genres.service.GenreService;
import com.example.demo.core.configuration.Constants;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;

import jakarta.validation.Valid;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.PutMapping;

@RestController
@RequestMapping(Constants.API_URL + "/book")
public class BookController {
    private final BookService bookService;
    private final GenreService genreService;
    private final ModelMapper modelMapper;

    public BookController(BookService bookService, GenreService genreService, ModelMapper modelMapper) {
        this.modelMapper = modelMapper;
        this.genreService = genreService;
        this.bookService = bookService;
    }

    private BookDto toDto(BookEntity entity) {
        return modelMapper.map(entity, BookDto.class);
    }

    private BookEntity toEntity(BookDto dto) {
        final BookEntity entity = modelMapper.map(dto, BookEntity.class);
        entity.setGenre(genreService.get(dto.getGenreId()));
        return entity;
    }

    @GetMapping
    public List<BookDto> getAll(@RequestParam(name = "genreId", defaultValue = "0") Integer genreId) {
        return bookService.getAll(genreId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public BookDto get(@PathVariable(name = "id") Integer id) {
        return toDto(bookService.get(id));
    }

    @PostMapping
    public BookDto create(@RequestBody @Valid BookDto dto) {
        return toDto(bookService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public BookDto update(@PathVariable(name = "id") Integer id, @RequestBody BookDto dto) {
        return toDto(bookService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public BookDto delete(@PathVariable(name = "id") Integer id) {
        return toDto(bookService.delete(id));
    }
}
