package com.example.demo.books.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.repository.BookRepository;

@Service
public class BookService {
    private final BookRepository repository;

    public BookService(BookRepository repository) {
        this.repository = repository;
    }

    public List<BookEntity> getAll(Integer genreId) {
        if (Objects.equals(genreId, 0)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(book -> book.getGenre().getId().equals(genreId))
                .toList();
    }

    public BookEntity get(Integer id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public BookEntity create(BookEntity entity) {
        return repository.create(entity);
    }

    public BookEntity update(Integer id, BookEntity entity) {
        final BookEntity existsEntity = get(id);
        existsEntity.setGenre(entity.getGenre());
        existsEntity.setName(entity.getName());
        existsEntity.setAuthor(entity.getAuthor());
        existsEntity.setYear(entity.getYear());
        existsEntity.setDescription(entity.getDescription());
        return repository.update(existsEntity);
    }

    public BookEntity delete(Integer id) {
        final BookEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }

    public void deleteAll() {
        repository.deleteAll();
    }
}
