package com.example.demo.genres.service;

import org.springframework.stereotype.Service;
import java.util.List;
import java.util.Optional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.genres.model.GenreEntity;

import com.example.demo.genres.repository.GenreRepository;

@Service
public class GenreService {
    private final GenreRepository repository;

    public GenreService(GenreRepository repository) {
        this.repository = repository;
    }

    public List<GenreEntity> getAll() {
        return repository.getAll();
    }

    public GenreEntity get(Integer id) {
        return Optional.ofNullable(repository.get(id)).orElseThrow(() -> new NotFoundException(id));
    }

    public GenreEntity create(GenreEntity entity) {
        return repository.create(entity);
    }

    public GenreEntity update(Integer id, GenreEntity entity) {
        final GenreEntity exisEntity = get(id);
        exisEntity.setName(entity.getName());
        return repository.update(exisEntity);
    }

    public GenreEntity delete(Integer id) {
        final GenreEntity existEntity = get(id);
        return repository.delete(existEntity);
    }

    public void deleteAll() {
        repository.deleteAll();
    }
}
