package com.example.demo.users.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.core.configuration.Constants;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/user")
public class UserController {
    private final UserService userService;
    private final ModelMapper modelMapper;
    private final BookService bookService;

    public UserController(UserService userService, ModelMapper modelMapper, BookService bookService) {
        this.modelMapper = modelMapper;
        this.userService = userService;
        this.bookService = bookService;
    }

    private UserDto toDto(UserEntity entity) {
        // return modelMapper.map(entity, UserDto.class);
        final var dto = new UserDto();
        dto.setId(entity.getId());
        dto.setUsername(entity.getUsername());
        dto.setPassword(entity.getPassword());
        dto.setIsAdmin(entity.getIsAdmin());
        dto.setBooks(entity.getBooks().stream().map(BookEntity::getId).toList());
        return dto;
    }

    private UserEntity toEntity(UserDto dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    @GetMapping()
    public List<UserDto> getAll() {
        return userService.getAll().stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public UserDto get(@PathVariable(name = "id") Integer id) {
        return toDto(userService.get(id));
    }

    @PostMapping
    public UserDto create(@Valid @RequestBody UserDto dto) {
        return toDto(userService.create(toEntity(dto)));
    }

    @PostMapping("/{id}/book/{bookId}")
    public UserDto addBook(@PathVariable(name = "id") Integer id, @PathVariable(name = "bookId") Integer bookId) {
        return toDto(userService.addBook(id, bookService.get(bookId)));
    }

    @PutMapping("/{id}")
    public UserDto update(@PathVariable(name = "id") Integer id, @RequestBody UserDto userDto) {
        return toDto(userService.update(id, toEntity(userDto)));
    }

    @DeleteMapping("/{id}")
    public UserDto delete(@PathVariable(name = "id") Integer id) {
        return toDto(userService.delete(id));
    }

    @DeleteMapping("/{id}/book/{bookId}")
    public UserDto deleteBook(@PathVariable(name = "id") Integer id, @PathVariable(name = "bookId") Integer bookId) {
        return toDto(userService.deleteBook(id, bookService.get(bookId)));
    }
}
