package com.example.demo.users.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.books.model.BookEntity;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;

@Service
public class UserService {
    private final UserRepository repository;

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    public List<UserEntity> getAll() {
        return repository.getAll();
    }

    public UserEntity get(Integer id) {
        return Optional.ofNullable(repository.get(id)).orElseThrow(() -> new NotFoundException(id));
    }

    public UserEntity create(UserEntity entity) {
        return repository.create(entity);
    }

    public UserEntity update(Integer id, UserEntity entity) {
        final UserEntity existedEntity = get(id);
        existedEntity.setUsername(entity.getUsername());
        existedEntity.setPassword(entity.getPassword());
        existedEntity.setIsAdmin(entity.getIsAdmin());
        return repository.update(existedEntity);
    }

    public UserEntity delete(Integer id) {
        final UserEntity existsentity = get(id);
        return repository.delete(existsentity);
    }

    public UserEntity addBook(Integer id, BookEntity book) {
        final UserEntity existedEntity = get(id);
        existedEntity.addBook(book);
        return existedEntity;
    }

    public UserEntity deleteBook(Integer id, BookEntity book) {
        final UserEntity existedEntity = get(id);
        existedEntity.deleteBook(book);
        return existedEntity;
    }

    public void deleteAll() {
        repository.deleteAll();
    }
}