package com.example.demo;

import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class BookServiceTests {
    @Autowired
    private BookService bookService;
    @Autowired
    private GenreService genreService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> bookService.get(0));
    }

    @Test
    @Order(1)
    void createTest() {
        genreService.deleteAll();
        bookService.deleteAll();

        genreService.create(new GenreEntity(null, "Роман"));
        genreService.create(new GenreEntity(null, "Комедия"));
        genreService.create(new GenreEntity(null, "Трагедия"));

        bookService.create(
                new BookEntity(null, genreService.get(1), "Гордость и предубеждение", "автор1", "год1", "описание"));

        bookService.create(new BookEntity(null, genreService.get(2), "Недоросль", "автор2", "год1", "описание"));

        final BookEntity last = bookService
                .create(new BookEntity(null, genreService.get(3), "Грозовой перевал", "автор3", "год1", "описание"));

        // проверка, что кол-во игр в хранилище равно кол-ву добавленных (проверка, что
        // все добавились)
        Assertions.assertEquals(3, bookService.getAll(0).size());
        // проверка, что в хранилище последняя игра та, которую добавили последней
        // (проверка, что всё правильно добавилось)
        Assertions.assertEquals(last, bookService.get(3));
    }

    @Test
    @Order(2)
    void update() {
        // на какой жанр будем менять (со Стратегии на FMV)
        final GenreEntity testEntity = genreService.get(1);

        // на какое название будем менять название игры
        final String testName = "Война и мир";

        // на какое название будем менять название игры
        final String testAuthor = "Л.Н.Толстой";

        // на какое название будем менять название игры
        final String testYear = "1892";

        // на какое описание будем менять описание игры
        final String testDescription = "Много французских букв";

        // получение последней в списке игры
        final BookEntity entity = bookService.get(3);

        // запоминание старого жанра игры
        final GenreEntity oldGenreEntity = entity.getGenre();

        // запоминание её названия до обновления
        final String oldName = entity.getName();

        // запоминание её цены до обновления
        final String oldAuthor = entity.getAuthor();

        // запоминание её описания до обновления
        final String oldDescription = entity.getDescription();

        // запоминание её описания до обновления
        final String oldYear = entity.getYear();

        // запоминание новых данных жанра
        final BookEntity newEntity = bookService.update(3,
                new BookEntity(1, testEntity, testName, testAuthor, testYear, testDescription));

        // Проверка, что кол-во игр до сих пор 3
        Assertions.assertEquals(3, bookService.getAll(0).size());

        Assertions.assertEquals(newEntity, bookService.get(3));

        // Проверка, что жанр поменялся
        Assertions.assertEquals(testEntity, newEntity.getGenre());
        Assertions.assertNotEquals(oldGenreEntity, newEntity.getGenre());

        // Проверка, что название поменялось
        Assertions.assertEquals(testName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());

        // Проверка, что название поменялось
        Assertions.assertEquals(testAuthor, newEntity.getAuthor());
        Assertions.assertNotEquals(oldAuthor, newEntity.getAuthor());

        // Проверка, что цена поменялась
        Assertions.assertEquals(testYear, newEntity.getYear());
        Assertions.assertNotEquals(oldYear, newEntity.getYear());

        // Проверка, что описание поменялось
        Assertions.assertEquals(testDescription, newEntity.getDescription());
        Assertions.assertNotEquals(oldDescription, newEntity.getDescription());
    }

    @Test
    @Order(3)
    void delete() {
        bookService.delete(3);
        // Проверка, что после удаления в хранилище осталось 2 игры
        Assertions.assertEquals(2, bookService.getAll(0).size());

        final BookEntity last = bookService.get(2);
        Assertions.assertEquals(2, last.getId());

        final BookEntity newEntity = bookService
                .create(new BookEntity(null, genreService.get(3), "книжка", "автор3", "год1", "описание"));
        // Проверка, что в хранилище теперь снова 3 игры
        Assertions.assertEquals(3, genreService.getAll().size());
        // Проверка, что id у добавленной игры стал 4
        Assertions.assertEquals(4, newEntity.getId());
    }
}
