package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;
import com.example.demo.core.error.NotFoundException;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class GenreServiceTests {
    @Autowired
    private GenreService genreService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> genreService.get(0));
    }

    @Test
    @Order(1)
    void createTest() {
        genreService.deleteAll();
        genreService.create(new GenreEntity(null, "Роман"));
        genreService.create(new GenreEntity(null, "Комедия"));
        final GenreEntity last = genreService.create(new GenreEntity(null, "Трагедия"));

        // Проверки, что значения равны. Первый параметр - эталон (ожидаемое значение),
        // второй - получаемое значение
        // проверка, что кол-во жанров в хранилище равно кол-ву добавленных (проверка,
        // что все добавились)

        Assertions.assertEquals(3, genreService.getAll().size());
        // проверка, что в хранилище последний жанр тот жанр, который добавили последним
        // (проверка, что всё правильно добавилось)
        Assertions.assertEquals(last, genreService.get(3));
    }

    @Test
    @Order(2)
    void update() {
        // на какое название будем менять название жанра
        final String test = "Антиутопия";
        // получение последнего в списке жанра
        final GenreEntity entity = genreService.get(3);
        // запоминание его названия до обновления
        final String oldName = entity.getName();
        // запоминание новых данных жанра
        final GenreEntity newEntity = genreService.update(3, new GenreEntity(1, test));
        // Проверка, что название поменялось
        Assertions.assertEquals(test, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    @Order(3)
    void delete() {
        genreService.delete(3);
        // Проверка, что после удаления в хранилище осталось 2 жанра
        Assertions.assertEquals(2, genreService.getAll().size());

        final GenreEntity newEntity = genreService.create(new GenreEntity(null, "Стратегия"));
        // Проверка, что в хранилище теперь снова 3 жанра
        Assertions.assertEquals(3, genreService.getAll().size());
        // Проверка, что id у добавленного жанра стал 4
        Assertions.assertEquals(4, newEntity.getId());
    }
}
