package com.example.demo;

import org.junit.jupiter.api.TestMethodOrder;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class UserServiceTests {
    @Autowired
    private UserService userService;
    @Autowired
    private BookService bookService;
    @Autowired
    private GenreService genreService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> userService.get(0));
    }

    @Test
    @Order(1)
    void createTest() {
        genreService.deleteAll();
        bookService.deleteAll();

        genreService.create(new GenreEntity(null, "Роман"));
        genreService.create(new GenreEntity(null, "Комедия"));
        genreService.create(new GenreEntity(null, "Трагедия"));

        bookService.create(new BookEntity(null, genreService.get(1), "Гордость и предубеждение", null, null, null));
        bookService.create(new BookEntity(null, genreService.get(2), "Недоросль", null, null, null));
        bookService.create(new BookEntity(null, genreService.get(3), "Грозовой перевал", null, null, null));

        userService.create(new UserEntity(null, "Пупа", "password", false));
        userService.create(new UserEntity(null, "Лупа", "password", false));

        Assertions.assertEquals(2, userService.getAll().size());
    }

    @Test
    @Order(2)
    void update() {
        // на какой ник будем менять ник пользователя
        final String testName = "Вася";

        // на какой пароль будем менять пароль пользователя
        final String testPassword = "TEST PASSWORD";

        // запоминание обновлённого пользователя
        final UserEntity newEntity = userService.update(1, new UserEntity(1, testName, testPassword, true));

        // Проверка, что кол-во пользователей до сих пор 2
        Assertions.assertEquals(2, userService.getAll().size());

        Assertions.assertEquals(newEntity, userService.get(1));

        // Проверка, что у пользователя нет игр
        Assertions.assertEquals(0, userService.get(1).getBooks().size());

        // Добавление пользователю 2 игр
        userService.addBook(1, bookService.get(3));
        userService.addBook(1, bookService.get(2));
        // Проверка, что у пользователя теперь есть 2 игры
        Assertions.assertEquals(2, userService.get(1).getBooks().size());

        // Удаление у пользователя игры
        userService.deleteBook(1, bookService.get(2));
        // Проверка, что у пользователя 1 игра
        Assertions.assertEquals(1, userService.get(1).getBooks().size());
    }

    @Test
    @Order(3)
    void delete() {
        userService.delete(1);
        // Проверка, что после удаления в хранилище осталcя пользователей
        Assertions.assertEquals(1, userService.getAll().size());

        userService.create(new UserEntity(null, "Пупа", "password", false));
        // Проверка, что в хранилище теперь снова 2 пользователя
        Assertions.assertEquals(2, userService.getAll().size());
    }
}
