package com.example.demo;

import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
    private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

    private final GenreService genreService;
    private final AuthorService authorService;
    private final BookService bookService;

    public DemoApplication(GenreService genreService, AuthorService authorService, BookService bookService) {
        this.genreService = genreService;
        this.authorService = authorService;
        this.bookService = bookService;
    }

    public static void main(String[] args) {
        SpringApplication.run(DemoApplication.class, args);
    }

    @Override
    public void run(String... args) throws Exception {
        if (args.length > 0 && Arrays.asList(args).contains("--populate")) {
            log.info("Create default genre values");
            final var genre1 = genreService.create(new GenreEntity("Роман"));
            final var genre2 = genreService.create(new GenreEntity("Комедия"));
            final var genre3 = genreService.create(new GenreEntity("Трагедия"));

            log.info("Create default author values");
            final var author1 = authorService.create(new AuthorEntity("Л.Н. Толстой"));
            final var author2 = authorService.create(new AuthorEntity("Илья Ильф"));
            final var author3 = authorService.create(new AuthorEntity("Ф.М. Достоевский"));

            log.info("Create default book values");
            final var books = List.of(
                    new BookEntity(genre1, author1, "", "", ""),
                    new BookEntity(genre1, author2, "", "", ""),
                    new BookEntity(genre2, author3, "", "", ""),
                    new BookEntity(genre2, author1, "", "", ""),
                    new BookEntity(genre3, author2, "", "", ""),
                    new BookEntity(genre3, author3, "", "", ""));
            books.forEach(book -> bookService.create(book));
        }
    }
}
