package com.example.demo.books.model;

import java.util.Objects;

import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.core.model.BaseEntity;
import com.example.demo.genres.model.GenreEntity;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;

@Entity
@Table(name = "books")
public class BookEntity extends BaseEntity {
    @ManyToOne
    @JoinColumn(name = "genreId", nullable = false)
    private GenreEntity genre;
    @ManyToOne
    @JoinColumn(name = "authorId", nullable = false)
    private AuthorEntity author;
    @Column(nullable = false, length = 50)
    private String name;
    @Column(name = "years", nullable = false, length = 4)
    private String year;
    @Column(nullable = false, length = 250)
    private String description;

    public BookEntity() {
    }

    public BookEntity(GenreEntity genre, AuthorEntity author, String name, String year, String description) {
        this.genre = genre;
        this.name = name;
        this.author = author;
        this.year = year;
        this.description = description;
    }

    public GenreEntity getGenre() {
        return genre;
    }

    public void setGenre(GenreEntity genre) {
        this.genre = genre;
    }

    public AuthorEntity getAuthor() {
        return author;
    }

    public void setAuthor(AuthorEntity author) {
        this.author = author;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getYear() {
        return year;
    }

    public void setYear(String year) {
        this.year = year;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, genre, name, author, year, description);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final BookEntity other = (BookEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getGenre(), genre)
                && Objects.equals(other.getName(), name)
                && Objects.equals(other.getAuthor(), author)
                && Objects.equals(other.getYear(), year)
                && Objects.equals(other.getDescription(), description);
    }
}
