package com.example.demo.favorites.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.favorites.model.FavoriteEntity;
import com.example.demo.favorites.repository.FavoriteRepository;

@Service
public class FavoriteService {
    private final FavoriteRepository repository;

    public FavoriteService(FavoriteRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<FavoriteEntity> getAll(Long userId) {
        if (userId == 0) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        return repository.findByUserId(userId);
    }

    @Transactional(readOnly = true)
    public FavoriteEntity get(Long id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(null, id));
    }

    @Transactional
    public FavoriteEntity create(FavoriteEntity entity) {
        return repository.save(entity);
    }

    @Transactional
    public FavoriteEntity update(Long id, FavoriteEntity entity) {
        final FavoriteEntity exisEntity = get(id);
        exisEntity.setUser(entity.getUser());
        exisEntity.setBook(entity.getBook());
        return repository.save(exisEntity);
    }

    @Transactional
    public FavoriteEntity delete(Long id) {
        final FavoriteEntity existEntity = get(id);
        repository.delete(existEntity);
        return existEntity;
    }
}
