package com.example.demo.genres.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/genre")
public class GenreController {
    private final GenreService genreService;
    private final ModelMapper modelMapper;

    public GenreController(GenreService genreService, ModelMapper modelMapper) {
        this.genreService = genreService;
        this.modelMapper = modelMapper;
    }

    private GenreDto toDto(GenreEntity entity) {
        return modelMapper.map(entity, GenreDto.class);
    }

    private GenreEntity toEntity(GenreDto dto) {
        return modelMapper.map(dto, GenreEntity.class);
    }

    @GetMapping
    public List<GenreDto> getAll() {
        return genreService.getAll().stream()
                .map(this::toDto)
                .toList();
    }

    @GetMapping("/{id}")
    public GenreDto get(@PathVariable(name = "id") Long id) {
        return toDto(genreService.get(id));
    }

    @PostMapping
    public GenreDto create(@RequestBody @Valid GenreDto dto) {
        return toDto(genreService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public GenreDto update(
            @PathVariable(name = "id") Long id,
            @RequestBody @Valid GenreDto dto) {
        return toDto(genreService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public GenreDto delete(@PathVariable(name = "id") Long id) {
        return toDto(genreService.delete(id));
    }
}
