package com.example.demo.genres.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.repository.GenreRepository;

@Service
public class GenreService {
    private final GenreRepository repository;

    public GenreService(GenreRepository repository) {
        this.repository = repository;
    }

    private void checkName(String name) {
        if (repository.findByNameIgnoreCase(name).isPresent()) {
            throw new IllegalArgumentException(
                    String.format("Genre with name %s is already exists", name));
        }
    }

    @Transactional(readOnly = true)
    public List<GenreEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public GenreEntity get(long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(GenreEntity.class, id));
    }

    @Transactional
    public GenreEntity create(GenreEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        checkName(entity.getName());
        return repository.save(entity);
    }

    @Transactional
    public GenreEntity update(Long id, GenreEntity entity) {
        final GenreEntity existsEntity = get(id);
        checkName(entity.getName());
        existsEntity.setName(entity.getName());
        return repository.save(existsEntity);
    }

    @Transactional
    public GenreEntity delete(Long id) {
        final GenreEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
