package com.example.demo.users.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;

@Service
public class UserService {
    private final UserRepository repository;

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(PageRequest pageRequest) {
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public UserEntity get(Long id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(null, id));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(Long id, UserEntity entity) {
        final UserEntity existsentity = get(id);
        existsentity.setUsername(entity.getUsername());
        existsentity.setPassword(entity.getPassword());
        existsentity.setIsAdmin(entity.getIsAdmin());
        return repository.save(existsentity);
    }

    @Transactional
    public UserEntity delete(Long id) {
        final UserEntity existsentity = get(id);
        repository.delete(existsentity);
        return existsentity;
    }

}