package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;

@SpringBootTest
public class AuthorServiceTests {
    @Autowired
    private AuthorService authorService;

    private AuthorEntity author;

    @BeforeEach
    void createData() {
        removeData();

        author = authorService.create(new AuthorEntity("А.С. Пушкин"));
        authorService.create(new AuthorEntity("А.П. Чехов"));

    }

    @AfterEach
    void removeData() {
        authorService.getAll().forEach(item -> authorService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> authorService.get(0));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(2, authorService.getAll().size());
        Assertions.assertEquals(author, authorService.get(author.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final AuthorEntity nonUniqueAuthor = new AuthorEntity("А.С. Пушкин");
        Assertions.assertThrows(IllegalArgumentException.class, () -> authorService.create(nonUniqueAuthor));
    }

    @Test
    void createNullableTest() {
        final AuthorEntity nullableAuthor = new AuthorEntity(null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> authorService.create(
                nullableAuthor));
    }

    @Test
    void updateTest() {
        final String newName = "TEST";
        final String oldName = author.getName();
        final AuthorEntity cat = new AuthorEntity(newName);
        final AuthorEntity newEntity = authorService.update(author.getId(), cat);
        Assertions.assertEquals(newName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    void deleteTest() {
        authorService.delete(author.getId());
        Assertions.assertEquals(1, authorService.getAll().size());

        Assertions.assertNotEquals(3, authorService.getAll().size());
    }
}
