package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

@SpringBootTest
class GenreServiceTests {
    @Autowired
    private GenreService genreService;

    private GenreEntity genre;

    @BeforeEach
    void createData() {
        removeData();

        genre = genreService.create(new GenreEntity("Drama"));
        genreService.create(new GenreEntity("Comedy"));

    }

    @AfterEach
    void removeData() {
        genreService.getAll().forEach(item -> genreService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> genreService.get(0));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(2, genreService.getAll().size());
        Assertions.assertEquals(genre, genreService.get(genre.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final GenreEntity nonUniqueGenre = new GenreEntity("Drama");
        Assertions.assertThrows(IllegalArgumentException.class, () -> genreService.create(nonUniqueGenre));
    }

    @Test
    void createNullableTest() {
        final GenreEntity nullableGenre = new GenreEntity(null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> genreService.create(
                nullableGenre));
    }

    @Test
    void updateTest() {
        final String newName = "TEST";
        final String oldName = genre.getName();
        final GenreEntity cat = new GenreEntity(newName);
        final GenreEntity newEntity = genreService.update(genre.getId(), cat);
        Assertions.assertEquals(newName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    void deleteTest() {
        genreService.delete(genre.getId());
        Assertions.assertEquals(1, genreService.getAll().size());

        Assertions.assertNotEquals(3, genreService.getAll().size());
    }
}
