package com.example.demo.authors.api;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.demo.core.configuration.Constants;
import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(AuthorController.URL)
public class AuthorController {
    public static final String URL = Constants.ADMIN_PREFIX + "/author";
    private static final String AUTHOR_VIEW = "author";
    private static final String AUTHOR_EDIT_VIEW = "author-edit";
    private static final String AUTHOR_ATTRIBUTE = "author";

    private final AuthorService authorService;
    private final ModelMapper modelMapper;

    public AuthorController(AuthorService authorService, ModelMapper modelMapper) {
        this.authorService = authorService;
        this.modelMapper = modelMapper;
    }

    private AuthorDto toDto(AuthorEntity entity) {
        return modelMapper.map(entity, AuthorDto.class);
    }

    private AuthorEntity toEntity(AuthorDto dto) {
        return modelMapper.map(dto, AuthorEntity.class);
    }

    @GetMapping
    public String getAll(Model model) {
        model.addAttribute(
                "items",
                authorService.getAll().stream()
                        .map(this::toDto)
                        .toList());
        return AUTHOR_VIEW;
    }

    @GetMapping("/edit/")
    public String create(Model model) {
        model.addAttribute(AUTHOR_ATTRIBUTE, new AuthorDto());
        return AUTHOR_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @ModelAttribute(name = AUTHOR_ATTRIBUTE) @Valid AuthorDto author,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return AUTHOR_EDIT_VIEW;
        }
        authorService.create(toEntity(author));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(AUTHOR_ATTRIBUTE, toDto(authorService.get(id)));
        return AUTHOR_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @ModelAttribute(name = AUTHOR_ATTRIBUTE) @Valid AuthorDto author,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return AUTHOR_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        authorService.update(id, toEntity(author));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id) {
        authorService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
