package com.example.demo.books.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.repository.BookRepository;

@Service
public class BookService {
    private final BookRepository repository;

    public BookService(BookRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<BookEntity> getAllBooks() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<BookEntity> getAllBooks(int page, int size) {
        return repository.findAll(PageRequest.of(page, size, Sort.by("id")));
    }

    @Transactional(readOnly = true)
    public List<BookEntity> getAll(long genreId, long authorId) {
        if (genreId == 0L && authorId == 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        } else if (genreId == 0L) {
            return repository.findByAuthorId(authorId);
        } else if (authorId == 0L) {
            return repository.findByGenreId(genreId);
        } else {
            return repository.findByGenreIdAndAuthorId(genreId, authorId);
        }
    }

    @Transactional(readOnly = true)
    public Page<BookEntity> getAll(long genreId, long authorId, int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);
        if (genreId == 0L && authorId == 0L) {
            return repository.findAll(pageRequest);
        } else if (genreId == 0L) {
            return repository.findByAuthorId(authorId, pageRequest);
        } else if (authorId == 0L) {
            return repository.findByGenreId(genreId, pageRequest);
        } else {
            return repository.findByGenreIdAndAuthorId(genreId, authorId, pageRequest);
        }
    }

    @Transactional(readOnly = true)
    public BookEntity get(long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(BookEntity.class, id));
    }

    @Transactional
    public BookEntity create(BookEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public BookEntity update(Long id, BookEntity entity) {
        final BookEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        existsEntity.setAuthor(entity.getAuthor());
        existsEntity.setGenre(entity.getGenre());
        existsEntity.setYear(entity.getYear());
        existsEntity.setDescription(entity.getDescription());
        return repository.save(existsEntity);
    }

    @Transactional
    public BookEntity delete(Long id) {
        final BookEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
