package com.example.demo.favorites.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.books.service.BookService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.favorites.model.FavoriteEntity;
import com.example.demo.favorites.repository.FavoriteRepository;
import com.example.demo.users.service.UserService;

@Service
public class FavoriteService {
    private final FavoriteRepository repository;
    private final UserService userService;
    private final BookService bookService;

    public FavoriteService(FavoriteRepository repository, UserService userService, BookService bookService) {
        this.repository = repository;
        this.userService = userService;
        this.bookService = bookService;
    }

    @Transactional(readOnly = true)
    public List<FavoriteEntity> getAll(Long userId) {
        if (userId == 0) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        return repository.findByUserId(userId);
    }

    @Transactional(readOnly = true)
    public Page<FavoriteEntity> getAll(Long userId, int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);
        if (userId == 0) {
            return repository.findAll(pageRequest);
        }
        return repository.findByUserId(userId, pageRequest);
    }

    // @Transactional(readOnly = true)
    // public FavoriteEntity get(Long id) {
    // return repository.findById(id).orElseThrow(() -> new NotFoundException(null,
    // id));
    // }

    @Transactional(readOnly = true)
    public FavoriteEntity get(long userId, long id) {
        userService.get(userId);
        return repository.findOneByUserIdAndId(userId, id)
                .orElseThrow(() -> new NotFoundException(FavoriteEntity.class, id));
    }

    @Transactional()
    public FavoriteEntity create(FavoriteEntity entity, Long userId) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }

        entity.setUser(userService.get(userId));
        return repository.save(entity);
    }

    @Transactional
    public FavoriteEntity toggle(Long userId, Long bookId, boolean isFavourite) {
        if (isFavourite)
            return delete(userId, bookId);

        return create(new FavoriteEntity(userService.get(userId), bookService.get(bookId)), userId);
    }

    @Transactional
    public FavoriteEntity update(long userId, Long id, FavoriteEntity entity) {
        final FavoriteEntity exisEntity = get(userId, id);
        exisEntity.setUser(entity.getUser());
        exisEntity.setBook(entity.getBook());
        return repository.save(exisEntity);
    }

    @Transactional
    public FavoriteEntity delete(long userId, Long id) {
        final FavoriteEntity existEntity = get(userId, id);
        repository.delete(existEntity);
        return existEntity;
    }
}
