package com.example.demo.genres.api;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.demo.core.configuration.Constants;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(GenreController.URL)
public class GenreController {
    public static final String URL = Constants.ADMIN_PREFIX + "/genre";
    private static final String GENRE_VIEW = "genre";
    private static final String GENRE_EDIT_VIEW = "genre-edit";
    private static final String GENRE_ATTRIBUTE = "genre";

    private final GenreService genreService;
    private final ModelMapper modelMapper;

    public GenreController(GenreService genreService, ModelMapper modelMapper) {
        this.genreService = genreService;
        this.modelMapper = modelMapper;
    }

    private GenreDto toDto(GenreEntity entity) {
        return modelMapper.map(entity, GenreDto.class);
    }

    private GenreEntity toEntity(GenreDto dto) {
        return modelMapper.map(dto, GenreEntity.class);
    }

    @GetMapping
    public String getAll(Model model) {
        model.addAttribute(
                "items",
                genreService.getAll().stream()
                        .map(this::toDto)
                        .toList());
        return GENRE_VIEW;
    }

    @GetMapping("/edit/")
    public String create(Model model) {
        model.addAttribute(GENRE_ATTRIBUTE, new GenreDto());
        return GENRE_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @ModelAttribute(name = GENRE_ATTRIBUTE) @Valid GenreDto genre,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return GENRE_EDIT_VIEW;
        }
        genreService.create(toEntity(genre));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(GENRE_ATTRIBUTE, toDto(genreService.get(id)));
        return GENRE_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @ModelAttribute(name = GENRE_ATTRIBUTE) @Valid GenreDto genre,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return GENRE_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        genreService.update(id, toEntity(genre));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id) {
        genreService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
