package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;

@SpringBootTest
class AuthorServiceTests {
    @Autowired
    private AuthorService authorService;

    private AuthorEntity author;

    @BeforeEach
    void createData() {
        removeData();

    }

    @AfterEach
    void removeData() {
    }

    @Test
    void getTest() {
        author = authorService.create(new AuthorEntity("Студия-дом"));
        Assertions.assertThrows(NotFoundException.class, () -> authorService.get(0L));
    }

    @Test
    void createTest() {
        author = authorService.create(new AuthorEntity("Дом"));
        Assertions.assertEquals(9, authorService.getAll().size());
        Assertions.assertEquals(author, authorService.get(author.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final AuthorEntity nonUniqueAuthor = new AuthorEntity("Студия");
        Assertions.assertThrows(IllegalArgumentException.class, () -> authorService.create(nonUniqueAuthor));
    }

    @Test
    void createNullableTest() {
        author = authorService.create(new AuthorEntity("Квартира"));
        final AuthorEntity nullableAuthor = new AuthorEntity(null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> authorService.create(nullableAuthor));
    }

    @Test
    void updateTest() {
        author = authorService.create(new AuthorEntity("Дом на колёсах"));
        final String test = "TEST";
        final String oldName = author.getName();
        final AuthorEntity newEntity = authorService.update(author.getId(), new AuthorEntity(test));
        Assertions.assertEquals(7, authorService.getAll().size());
        Assertions.assertEquals(newEntity, authorService.get(author.getId()));
        Assertions.assertEquals(test, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    void deleteTest() {
        author = authorService.create(new AuthorEntity("2х ярусная квартира"));
        authorService.delete(author.getId());
        Assertions.assertEquals(9, authorService.getAll().size());

        final AuthorEntity newEntity = authorService.create(new AuthorEntity(author.getName()));
        Assertions.assertEquals(10, authorService.getAll().size());
        Assertions.assertNotEquals(author.getId(), newEntity.getId());
    }
}