package com.example.demo;

import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.authors.model.AuthorEntity;
import com.example.demo.authors.service.AuthorService;
import com.example.demo.books.model.BookEntity;
import com.example.demo.books.service.BookService;
import com.example.demo.favorites.model.FavoriteEntity;
import com.example.demo.favorites.service.FavoriteService;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserRole;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class FavoriteServiceTests {

    @Autowired
    private FavoriteService favoriteService;

    private FavoriteEntity lastFavorite;
    private FavoriteEntity lastFavorite2;

    private GenreEntity genre;
    private AuthorEntity author;
    private BookEntity book1;
    private BookEntity book2;
    private UserEntity user;

    @Autowired
    private GenreService genreService;

    @Autowired
    private AuthorService authorService;

    @Autowired
    private BookService bookService;

    @Autowired
    private UserService userService;

    @BeforeEach
    void createData() {
        removeData();

        genre = genreService.create(new GenreEntity("Трагедия"));
        author = authorService.findByName("И.С. Тургенев");
        if (author == null) {
            author = authorService.create(new AuthorEntity("И.С. Тургенев"));
        }

        book1 = bookService.create(new BookEntity(genre, author, "какой-то фильм",
                "1", "1"));
        book2 = bookService.create(new BookEntity(genre, author, "2", "2", "2"));

        user = userService.create(new UserEntity("elina", "123"));
        user.setRole(UserRole.ADMIN);

        lastFavorite = favoriteService.create(new FavoriteEntity(user, book1), user.getId());
        lastFavorite2 = favoriteService.create(new FavoriteEntity(user, book2), user.getId());
    }

    @AfterEach
    void removeData() {
        favoriteService.getAll(0L).forEach(fv -> favoriteService.delete(user.getId(), fv.getId()));
        userService.getAll().forEach(u -> userService.delete(u.getId()));
        bookService.getAll(0, 0).forEach(item -> bookService.delete(item.getId()));
        genreService.getAll().forEach(item -> genreService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NullPointerException.class, () -> favoriteService.get(user.getId(), 0L)); // NotFoundException
    }

    @Test
    void createTest() {
        Assertions.assertEquals(2, favoriteService.getAll(user.getId()).size());
    }

    @Test
    void update() {
        final FavoriteEntity fav = favoriteService.create(new FavoriteEntity(user, book2), user.getId());

        final FavoriteEntity oldFav = favoriteService.get(user.getId(), lastFavorite.getId());

        final FavoriteEntity newFav = favoriteService.update(user.getId(), lastFavorite2.getId(),
                fav);

        Assertions.assertNotEquals(oldFav.getBook().getName(),
                newFav.getBook().getName());

    }

    @Test

    @Order(3)
    void delete() {

        favoriteService.delete(user.getId(), lastFavorite.getId());

        Assertions.assertEquals(1, favoriteService.getAll(user.getId()).size());
    }
}
