package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.genres.model.GenreEntity;
import com.example.demo.genres.service.GenreService;

@SpringBootTest
class GenreServiceTests {
    @Autowired
    private GenreService genreService;

    private GenreEntity genre;

    @BeforeEach
    void createData() {
        removeData();

    }

    @AfterEach
    void removeData() {
    }

    @Test
    void getTest() {
        genre = genreService.create(new GenreEntity("Студия-дом"));
        Assertions.assertThrows(NotFoundException.class, () -> genreService.get(0L));
    }

    @Test
    void createTest() {
        genre = genreService.create(new GenreEntity("Дом"));
        Assertions.assertEquals(9, genreService.getAll().size());
        Assertions.assertEquals(genre, genreService.get(genre.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final GenreEntity nonUniqueGenre = new GenreEntity("Студия");
        Assertions.assertThrows(IllegalArgumentException.class, () -> genreService.create(nonUniqueGenre));
    }

    @Test
    void createNullableTest() {
        genre = genreService.create(new GenreEntity("Квартира"));
        final GenreEntity nullableGenre = new GenreEntity(null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> genreService.create(nullableGenre));
    }

    @Test
    void updateTest() {
        genre = genreService.create(new GenreEntity("Дом на колёсах"));
        final String test = "TEST";
        final String oldName = genre.getName();
        final GenreEntity newEntity = genreService.update(genre.getId(), new GenreEntity(test));
        Assertions.assertEquals(7, genreService.getAll().size());
        Assertions.assertEquals(newEntity, genreService.get(genre.getId()));
        Assertions.assertEquals(test, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    void deleteTest() {
        genre = genreService.create(new GenreEntity("2х ярусная квартира"));
        genreService.delete(genre.getId());
        Assertions.assertEquals(9, genreService.getAll().size());

        final GenreEntity newEntity = genreService.create(new GenreEntity(genre.getName()));
        Assertions.assertEquals(10, genreService.getAll().size());
        Assertions.assertNotEquals(genre.getId(), newEntity.getId());
    }
}